// ------------------------------------
//	gba.h by eloist modified by dennis
// ------------------------------------

#ifndef _GBA_H_
#define _GBA_H_

// -------------------
//	standard typedefs
// -------------------

typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned long u32;

typedef signed char s8;
typedef signed short s16;
typedef signed long s32;

typedef unsigned char byte;
typedef unsigned short hword;
typedef unsigned long word;

// ------------------
//	GBA memory areas
// ------------------

#define BG_PALETTE_MEM	((u16*)0x05000000)
#define OBJ_PALETTE_MEM	((u16*)0x05000200)
#define VRAM			(u16*)0x06000000
#define OAM_DATA		(u16*)0x06010000
#define OAM_MEM			(u32*)0x07000000
#define SRAM			((u8*)0x0E000000)

// -----------
//	registers
// -----------

#define REG_INTERRUPT	*(volatile u32*)0x03007FFC

#define REG_DISPCNT		*(volatile u16*)0x04000000
#define REG_DISPSTAT	*(volatile u16*)0x04000004
#define REG_VCOUNT		*(volatile u16*)0x04000006
#define REG_BG0CNT		*(volatile u16*)0x04000008
#define REG_BG1CNT		*(volatile u16*)0x0400000A
#define REG_BG2CNT		*(volatile u16*)0x0400000C
#define REG_BG3CNT		*(volatile u16*)0x0400000E
#define REG_BG0HOFS		*(volatile u16*)0x04000010
#define REG_BG0VOFS		*(volatile u16*)0x04000012
#define REG_BG1HOFS		*(volatile u16*)0x04000014
#define REG_BG1VOFS		*(volatile u16*)0x04000016
#define REG_BG2HOFS		*(volatile u16*)0x04000018
#define REG_BG2VOFS		*(volatile u16*)0x0400001A
#define REG_BG3HOFS		*(volatile u16*)0x0400001C
#define REG_BG3VOFS		*(volatile u16*)0x0400001E
#define REG_BG2PA		*(volatile u16*)0x04000020
#define REG_BG2PB		*(volatile u16*)0x04000022
#define REG_BG2PC		*(volatile u16*)0x04000024
#define REG_BG2PD		*(volatile u16*)0x04000026
#define REG_BG2X		*(volatile u32*)0x04000028
#define REG_BG2X_L		*(volatile u16*)0x04000028
#define REG_BG2X_H		*(volatile u16*)0x0400002A
#define REG_BG2Y		*(volatile u32*)0x0400002C
#define REG_BG2Y_L		*(volatile u16*)0x0400002C
#define REG_BG2Y_H		*(volatile u16*)0x0400002E
#define REG_BG3PA		*(volatile u16*)0x04000030
#define REG_BG3PB		*(volatile u16*)0x04000032
#define REG_BG3PC		*(volatile u16*)0x04000034
#define REG_BG3PD		*(volatile u16*)0x04000036
#define REG_BG3X		*(volatile u32*)0x04000038
#define REG_BG3X_L		*(volatile u16*)0x04000038
#define REG_BG3X_H		*(volatile u16*)0x0400003A
#define REG_BG3Y		*(volatile u32*)0x0400003C
#define REG_BG3Y_L		*(volatile u16*)0x0400003C
#define REG_BG3Y_H		*(volatile u16*)0x0400003E
#define REG_WIN0H		*(volatile u16*)0x04000040
#define REG_WIN1H		*(volatile u16*)0x04000042
#define REG_WIN0V		*(volatile u16*)0x04000044
#define REG_WIN1V		*(volatile u16*)0x04000046
#define REG_WININ		*(volatile u16*)0x04000048
#define REG_WINOUT		*(volatile u16*)0x0400004A
#define REG_MOSAIC		*(volatile u32*)0x0400004C
#define REG_MOSAIC_L	*(volatile u32*)0x0400004C
#define REG_MOSAIC_H	*(volatile u32*)0x0400004E
#define REG_BLDMOD		*(volatile u16*)0x04000050
#define REG_COLEV		*(volatile u16*)0x04000052
#define REG_COLEY		*(volatile u16*)0x04000054
#define REG_SG10		*(volatile u32*)0x04000060
#define REG_SG10_L		*(volatile u16*)0x04000060
#define REG_SG10_H		*(volatile u16*)0x04000062
#define REG_SG11		*(volatile u16*)0x04000064
#define REG_SG20		*(volatile u16*)0x04000068
#define REG_SG21		*(volatile u16*)0x0400006C
#define REG_SG30		*(volatile u32*)0x04000070
#define REG_SG30_L		*(volatile u16*)0x04000070
#define REG_SG30_H		*(volatile u16*)0x04000072
#define REG_SG31		*(volatile u16*)0x04000074
#define REG_SG40		*(volatile u16*)0x04000078
#define REG_SG41		*(volatile u16*)0x0400007C
#define REG_SGCNT0		*(volatile u32*)0x04000080
#define REG_SGCNT0_L	*(volatile u16*)0x04000080
#define REG_SGCNT0_H	*(volatile u16*)0x04000082
#define REG_SGCNT1		*(volatile u16*)0x04000084
#define REG_SGBIAS		*(volatile u16*)0x04000088
#define REG_SGWR0		*(volatile u32*)0x04000090
#define REG_SGWR0_L		*(volatile u16*)0x04000090
#define REG_SGWR0_H		*(volatile u16*)0x04000092
#define REG_SGWR1		*(volatile u32*)0x04000094
#define REG_SGWR1_L		*(volatile u16*)0x04000094
#define REG_SGWR1_H		*(volatile u16*)0x04000096
#define REG_SGWR2		*(volatile u32*)0x04000098
#define REG_SGWR2_L		*(volatile u16*)0x04000098
#define REG_SGWR2_H		*(volatile u16*)0x0400009A
#define REG_SGWR3		*(volatile u32*)0x0400009C
#define REG_SGWR3_L		*(volatile u16*)0x0400009C
#define REG_SGWR3_H		*(volatile u16*)0x0400009E
#define REG_SGFIF0A		*(volatile u32*)0x040000A0
#define REG_SGFIFOA_L	*(volatile u16*)0x040000A0
#define REG_SGFIFOA_H	*(volatile u16*)0x040000A2
#define REG_SGFIFOB		*(volatile u32*)0x040000A4
#define REG_SGFIFOB_L	*(volatile u16*)0x040000A4
#define REG_SGFIFOB_H	*(volatile u16*)0x040000A6
#define REG_DM0SAD		*(volatile u32*)0x040000B0
#define REG_DM0SAD_L	*(volatile u16*)0x040000B0
#define REG_DM0SAD_H	*(volatile u16*)0x040000B2
#define REG_DM0DAD		*(volatile u32*)0x040000B4
#define REG_DM0DAD_L	*(volatile u16*)0x040000B4
#define REG_DM0DAD_H	*(volatile u16*)0x040000B6
#define REG_DM0CNT		*(volatile u32*)0x040000B8
#define REG_DM0CNT_L	*(volatile u16*)0x040000B8
#define REG_DM0CNT_H	*(volatile u16*)0x040000BA
#define REG_DM1SAD		*(volatile u32*)0x040000BC
#define REG_DM1SAD_L	*(volatile u16*)0x040000BC
#define REG_DM1SAD_H	*(volatile u16*)0x040000BE
#define REG_DM1DAD		*(volatile u32*)0x040000C0
#define REG_DM1DAD_L	*(volatile u16*)0x040000C0
#define REG_DM1DAD_H	*(volatile u16*)0x040000C2
#define REG_DM1CNT		*(volatile u32*)0x040000C4
#define REG_DM1CNT_L	*(volatile u16*)0x040000C4
#define REG_DM1CNT_H	*(volatile u16*)0x040000C6
#define REG_DM2SAD		*(volatile u32*)0x040000C8
#define REG_DM2SAD_L	*(volatile u16*)0x040000C8
#define REG_DM2SAD_H	*(volatile u16*)0x040000CA
#define REG_DM2DAD		*(volatile u32*)0x040000CC
#define REG_DM2DAD_L	*(volatile u16*)0x040000CC
#define REG_DM2DAD_H	*(volatile u16*)0x040000CE
#define REG_DM2CNT		*(volatile u32*)0x040000D0
#define REG_DM2CNT_L	*(volatile u16*)0x040000D0
#define REG_DM2CNT_H	*(volatile u16*)0x040000D2
#define REG_DM3SAD		*(volatile u32*)0x040000D4
#define REG_DM3SAD_L	*(volatile u16*)0x040000D4
#define REG_DM3SAD_H	*(volatile u16*)0x040000D6
#define REG_DM3DAD		*(volatile u32*)0x040000D8
#define REG_DM3DAD_L	*(volatile u16*)0x040000D8
#define REG_DM3DAD_H	*(volatile u16*)0x040000DA
#define REG_DM3CNT		*(volatile u32*)0x040000DC
#define REG_DM3CNT_L	*(volatile u16*)0x040000DC
#define REG_DM3CNT_H	*(volatile u16*)0x040000DE
#define REG_TM0D		*(volatile u16*)0x04000100
#define REG_TM0CNT		*(volatile u16*)0x04000102
#define REG_TM1D		*(volatile u16*)0x04000104
#define REG_TM1CNT		*(volatile u16*)0x04000106
#define REG_TM2D		*(volatile u16*)0x04000108
#define REG_TM2CNT		*(volatile u16*)0x0400010A
#define REG_TM3D		*(volatile u16*)0x0400010C
#define REG_TM3CNT		*(volatile u16*)0x0400010E
#define REG_SCD0		*(volatile u16*)0x04000120
#define REG_SCD1		*(volatile u16*)0x04000122
#define REG_SCD2		*(volatile u16*)0x04000124
#define REG_SCD3		*(volatile u16*)0x04000126
#define REG_SCCNT		*(volatile u32*)0x04000128
#define REG_SIOCNT		*(volatile u16*)0x04000128
#define REG_SCCNT_L		*(volatile u16*)0x04000128
#define REG_SCCNT_H		*(volatile u16*)0x0400012A
#define REG_SIODATA8	*(volatile u16*)0x0400012A
#define REG_P1			*(volatile u16*)0x04000130
#define REG_P1CNT		*(volatile u16*)0x04000132
#define REG_R			*(volatile u16*)0x04000134
#define REG_RCNT		*(volatile u16*)0x04000134
#define REG_HS_CTRL		*(volatile u16*)0x04000140
#define REG_JOYRE		*(volatile u32*)0x04000150
#define REG_JOYRE_L		*(volatile u16*)0x04000150
#define REG_JOYRE_H		*(volatile u16*)0x04000152
#define REG_JOYTR		*(volatile u32*)0x04000154
#define REG_JOYTR_L		*(volatile u16*)0x04000154
#define REG_JOYTR_H		*(volatile u16*)0x04000156
#define REG_JSTAT		*(volatile u32*)0x04000158
#define REG_JSTAT_L		*(volatile u16*)0x04000158
#define REG_JSTAT_H		*(volatile u16*)0x0400015A
#define REG_IE			*(volatile u16*)0x04000200
#define REG_IF			*(volatile u16*)0x04000202
#define REG_WSCNT		*(volatile u16*)0x04000204
#define REG_IME			*(volatile u16*)0x04000208
#define REG_PAUSE		*(volatile u16*)0x04000300

// -------------------
//	REG_DISPCNT flags
// -------------------

#define DISP_MODE_0		0x0000
#define DISP_MODE_1		0x0001
#define DISP_MODE_2		0x0002
#define DISP_MODE_3		0x0003
#define DISP_MODE_4		0x0004
#define DISP_MODE_5		0x0005
					//	0x0006 - invalid
					//	0x0007 - invalid
#define DISP_GB_CART	0x0008

#define DISP_DB_ON		0x0010
#define DISP_HB_ON		0x0020
#define DISP_OM_1D		0x0040
#define DISP_LCDC_OFF	0x0080

#define DISP_BG0_ON		0x0100
#define DISP_BG1_ON		0x0200
#define DISP_BG2_ON		0x0400
#define DISP_BG3_ON		0x0800

#define DISP_OBJ_ON		0x1000
#define DISP_W0_ON		0x2000
#define DISP_W1_ON		0x3000
#define DISP_OW_ON		0x4000

// --------------------
//	BGCNT flags
// --------------------

#define BGCNT_PRIORITY_0	0x0000
#define BGCNT_PRIORITY_1	0x0001
#define BGCNT_PRIORITY_2	0x0002
#define BGCNT_PRIORITY_3	0x0003

#define BGCNT_TILEDATA_0000	0x0000
#define BGCNT_TILEDATA_4000	0x0004
#define BGCNT_TILEDATA_8000	0x0008
#define BGCNT_TILEDATA_C000	0x000C

#define BGCNT_MOSIAC_ON		0x0040
#define BGCNT_256_COLOR		0x0080

#define BGCNT_TILEMAP_0000	0x0000
#define BGCNT_TILEMAP_0800	0x0100
#define BGCNT_TILEMAP_1000	0x0200
#define BGCNT_TILEMAP_1800	0x0300
#define BGCNT_TILEMAP_2000	0x0400
#define BGCNT_TILEMAP_2800	0x0500
#define BGCNT_TILEMAP_3000	0x0600
#define BGCNT_TILEMAP_3800	0x0700
#define BGCNT_TILEMAP_4000	0x0800
#define BGCNT_TILEMAP_4800	0x0900
#define BGCNT_TILEMAP_5000	0x0a00
#define BGCNT_TILEMAP_5800	0x0b00
#define BGCNT_TILEMAP_6000	0x0c00
#define BGCNT_TILEMAP_6800	0x0d00
#define BGCNT_TILEMAP_7000	0x0e00
#define BGCNT_TILEMAP_7800	0x0f00
#define BGCNT_TILEMAP_8000	0x1000
#define BGCNT_TILEMAP_8800	0x1100
#define BGCNT_TILEMAP_9000	0x1200
#define BGCNT_TILEMAP_9800	0x1300
#define BGCNT_TILEMAP_A000	0x1400
#define BGCNT_TILEMAP_A800	0x1500
#define BGCNT_TILEMAP_B000	0x1600
#define BGCNT_TILEMAP_B800	0x1700
#define BGCNT_TILEMAP_C000	0x1800
#define BGCNT_TILEMAP_C800	0x1900
#define BGCNT_TILEMAP_D000	0x1a00
#define BGCNT_TILEMAP_D800	0x1b00
#define BGCNT_TILEMAP_E000	0x1c00
#define BGCNT_TILEMAP_E800	0x1d00
#define BGCNT_TILEMAP_F000	0x1e00
#define BGCNT_TILEMAP_F800	0x1f00

#define BGCNT_AREA_OVERFLOW	0x2000

#define BGCNT_32x32_MAP		0x0000
#define BGCNT_64x32_MAP		0x4000
#define BGCNT_32x64_MAP		0x8000
#define BGCNT_64x64_MAP		0xc000

#define BGCNT_16x_MAP		0x0000
#define BGCNT_32x_MAP		0x4000
#define BGCNT_64x_MAP		0x8000
#define BGCNT_128x_MAP		0xc000

// ----------------------------
//	OAM structs
// ----------------------------

//Atribute0 stuff
#define SPRITE_ROTATION_FLAG	0x0100
#define SPRITE_SIZE_DOUBLE		0x0200
#define SPRITE_MODE_NORMAL		0x0000
#define SPRITE_MODE_TRANSPERANT	0x0400
#define SPRITE_MODE_WINDOWED	0x0800
#define SPRITE_MOSAIC			0x1000
#define SPRITE_16_COLOR			0x0000
#define SPRITE_256_COLOR		0x2000
#define SPRITE_SQUARE			0x0000
#define SPRITE_TALL				0x8000
#define SPRITE_WIDE				0x4000

//Atribute1 stuff
#define SPRITE_ROTDATA(n)		((n) << 9)
#define SPRITE_HORIZONTAL_FLIP	0x1000
#define SPRITE_VERTICAL_FLIP	0x2000
#define SPRITE_SIZE_8			0x0000
#define SPRITE_SIZE_16			0x4000
#define SPRITE_SIZE_32			0x8000
#define SPRITE_SIZE_64			0xC000

//atribute2 stuff
#define SPRITE_PRIORITY(n)		((n) << 10)
#define SPRITE_PALETTE(n)		((n) << 12)

typedef struct
{
	u16 attribute0;
	u16 attribute1;
	u16 attribute2;
	u16 attribute3;
	
} OAMEntry;

//Another struct for rotation data
typedef struct
{
	u16 filler1[3];
	u16 pa;
	u16 filler2[3];
	u16 pb; 
	u16 filler3[3];
	u16 pc;
	u16 filler4[3];
	u16 pd;

} OAMRotData;

#define DMA_ENABLE 0x80000000
#define DMA_INTERUPT_ENABLE 0x40000000
#define DMA_TIMING_IMMEDIATE 0x00000000
#define DMA_TIMING_VBLANK 0x10000000
#define DMA_TIMING_HBLANK 0x20000000
#define DMA_TIMING_SYNC_TO_DISPLAY 0x30000000
#define DMA_16 0x00000000
#define DMA_32 0x04000000
#define DMA_REPEATE 0x02000000
#define DMA_SOURCE_INCREMENT 0x00000000
#define DMA_SOURCE_DECREMENT 0x00800000
#define DMA_SOURCE_FIXED 0x01000000
#define DMA_DEST_INCREMENT 0x00000000
#define DMA_DEST_DECREMENT 0x00200000
#define DMA_DEST_FIXED 0x00400000
#define DMA_DEST_RELOAD 0x00600000

// these defines group common options to save typing.

#define DMA_32NOW DMA_ENABLE | DMA_TIMING_IMMEDIATE |DMA_32 
#define DMA_16NOW DMA_ENABLE | DMA_TIMING_IMMEDIATE |DMA_16

#define KEY_BUTTON_MASK             0x030f      // Button
#define KEY_PLUS_MASK           0x00f0      // +Control Pad
#define KEY_ALL_MASK            0x03ff      // All Key

#define KEY_A_BUTTON                0x0001      // A Button
#define KEY_B_BUTTON                0x0002      // B Button
#define KEY_SELECT_BUTTON           0x0004      // SELECT
#define KEY_START_BUTTON            0x0008      // START
#define KEY_R                   0x0010      // Right Button
#define KEY_L                   0x0020      // Left Button
#define KEY_U                   0x0040      // Up Button
#define KEY_D                   0x0080      // Down Button
#define KEY_R_BUTTON                0x0100      // R Button
#define KEY_L_BUTTON                0x0200      // L Button

#define KEY_RESET				(KEY_A_BUTTON | KEY_B_BUTTON | KEY_START_BUTTON | KEY_SELECT_BUTTON)

#define KEY_IF_ENABLE           0x4000      // Interrupt Request Enable
#define KEY_OR_INTR             0x0000      // Normal Key Interrupt
#define KEY_AND_INTR            0x8000      // AND Key Interrupt

// ------------------
//	window registers
// ------------------

#define WIN0_BG0				0x0001
#define WIN0_BG1				0x0002
#define WIN0_BG2				0x0004
#define WIN0_BG3				0x0008
#define WIN0_SPRITES			0x0010
#define WIN0_BLENDS				0x0020
#define WIN1_BG0				0x0100
#define WIN1_BG1				0x0200
#define WIN1_BG2				0x0400
#define WIN1_BG3				0x0800
#define WIN1_SPRITES			0x1000
#define WIN1_BLENDS				0x2000

#define INT_VBLANK 0x0001
#define INT_HBLANK 0x0002 
#define INT_VCOUNT 0x0004 //you can set the display to generate an interrupt when it reaches a particular line on the screen
#define INT_TIMMER0 0x0008
#define INT_TIMMER1 0x0010
#define INT_TIMMER2 0x0020 
#define INT_TIMMER3 0x0040
#define INT_COMUNICATION 0x0080 //serial communication interupt
#define INT_DMA0 0x0100
#define INT_DMA1 0x0200
#define INT_DMA2 0x0400
#define INT_DMA3 0x0800
#define INT_KEYBOARD 0x1000
#define INT_CART 0x2000 //the cart can actually generate an interupt

// --------------------
//	utility functions
// --------------------

/************************************************************\
* The GBA stores 15 bit color in blue green red format meaning
* the first 5 bits are the red the next 5 bits are green and the next
* 5 blue. This macro just shifts the colors over the correct amount.
* RGB is the more standard way to represent color. 
\************************************************************/
#define RGB(r,g,b) ((r)+((g)<<5)+((b)<<10)) 
#define BGR(b,g,r) ((r)+((g)<<5)+((b)<<10)) 

#define waitForVSync() { while(REG_VCOUNT != 160); }

#endif /* _GBA_H_ */
