// --------
//	main.c
// --------

#include "gba.h"
#include "font_lz.h"
//#include "mbv2lib.h"

//#include <stdio.h>

#define FONT_TILESET		((u32*)VRAM)
#define	BG0_MAP				((u16*)0x0600c000)

extern void VBlankIntrWait(void);
extern void LZ77UnCompVram(void *Srcp, void *Destp);

extern void interruptHandler(void);

extern void __FarProcedure (void (*ptr)(), ...);  // Reference to routine in crt0.S
extern void setRamStart(u32 a) __attribute__ ((section (".iwram")));
static void set_ram_start(int i)
{
    __FarProcedure(setRamStart, i);
	
} // of set_ram_start()

static void loadFont(void)
{
	LZ77UnCompVram((void *)font_LZ,(void *)FONT_TILESET);

	BG_PALETTE_MEM[1]  = RGB(16, 16, 16);
	BG_PALETTE_MEM[2]  = RGB(31, 31, 31);

	BG_PALETTE_MEM[17] = RGB(0,  16, 0);
	BG_PALETTE_MEM[18] = RGB(0,  31, 0);

} // of loadFont()

static void print(u8 x, u8 y, u16 pal, const char *string)
{
	u16 *map = BG0_MAP;
	
	map += (y * 32) + x;
	pal <<= 12;
	
	while(*string != '\0') *map++ = ((*string++) - 32) | pal;

} // of print()

#if 0
static void flip(void)
{
	static int flipper = 0;
	static int count = 0;
	static char *flipperChars = "0O";
	
	if(count++ == 10)
	{
		BG0_MAP[29] = flipperChars[flipper++] - 32;
		if(flipper == 2) flipper = 0;
		count = 0;
	}

} // of flip()
#endif

static void clearStatusArea(void)
{
	int i;

	for(i = 0; i < (6 * 32); i++)
		BG0_MAP[(10 * 32) + i] = 0;

} // of clearStatusArea()

#if 0
static void dumpSRAM(void)
{
	int bank;

	clearStatusArea();
	print(1, 10, 0, "DUMPING SRAM TO PC...  ");

	for(bank = 0; bank < 4; bank++)
	{
		FILE f;
		char fname[10];
		int i;

		print(3, 11 + bank, 0, "DUMPING BANK   ...");
		BG0_MAP[(11 + bank) * 32 + 16] = '1' - 32 + bank;

		set_ram_start(bank);
		
		sprintf(fname, "bank%d.sav", bank);
		
		f = dfopen(fname, "wb");
		
		for(i = 0; i <= 0xffff; i++)
		{
			if((i & 0xff) == 0)	flip();
			dfputc(SRAM[i], f);
		}

		dfclose(f);

		print(22, 11 + bank, 1, "DONE!");
	}

} // of dumpSRAM()

static void readSRAM(void) 
{
	clearStatusArea();
	print(1, 10, 0, "READING SRAM FROM PC...");

} // of readSRAM()
#endif

static void blankSRAM(void)
{
	int bank, i;
	
	clearStatusArea();
	print(1, 10, 0, "ERASING SRAM...");

	for(bank = 0; bank < 4; bank++)
	{
		set_ram_start(bank);
			
		print(3, 11 + bank, 0, "ERASING BANK   ...");
		BG0_MAP[(11 + bank) * 32 + 16] = '1' - 32 + bank;

		for(i = 0; i <= 0xffff; i++)
		{
//			if((i & 0xff) == 0) flip();
			SRAM[i] = 0;
		}

		print(22, 11 + bank, 1, "DONE!");
	}
	
} // of blankSRAM()

void AgbMain(void)
{
	// setup our interrupt handler
	REG_INTERRUPT = (u32)interruptHandler;
	REG_DISPSTAT |= 0x8;
	REG_IE = INT_VBLANK;
	REG_IME = 1;
	
	loadFont();
	REG_BG0CNT = BGCNT_TILEMAP_C000 | BGCNT_TILEDATA_0000 | BGCNT_PRIORITY_0;
	REG_BG0VOFS = -4;
	REG_DISPCNT = DISP_MODE_0 | DISP_BG0_ON;

	print(1, 0, 0, "SRAM UTILITY 0.1");

//	print(1, 3, 0, "A   - DUMP TO PC");
//	print(1, 5, 0, "B   - READ FROM PC");
	print(1, 3, 0, "L/R - ERASE SRAM");

	for( ; ; )
	{
		u16 keys = ~REG_P1 & KEY_ALL_MASK;
/*
		if(keys == KEY_A_BUTTON)
		{
			while(~REG_P1 & KEY_ALL_MASK) VBlankIntrWait();
			dumpSRAM();
		}

		else if(keys == KEY_B_BUTTON)
		{
			while(~REG_P1 & KEY_ALL_MASK) VBlankIntrWait();
			readSRAM();
		}
*/
		if(keys == (KEY_L_BUTTON | KEY_R_BUTTON))
		{
			while(~REG_P1 & KEY_ALL_MASK) VBlankIntrWait();
			blankSRAM();
		}
		
		VBlankIntrWait();
	}

} // of AgbMain()
